(function () {
  // Get CKEditor Plugin base class from Drupal’s global CKEditor5 loader
  const Plugin = window.CKEditor5.core.Plugin;
  const elements = ['table', 'thead', 'tbody', 'tfoot', 'tr', 'td', 'th'];

  class CKEditor5TableFixPlugin extends Plugin {
    static get pluginName() {
      return 'ckeditor5_table_fix_plugin';
    }

    init() {
      const editor = this.editor;
      if (!editor) {
        console.error('Editor is undefined in init()');
        return;
      }

      // Register all table-related elements
      editor.model.schema.register('table', {
        allowIn: '$root',
        allowContentOf: '$block',
        isBlock: true,
      });

      for (const tag of ['thead', 'tbody', 'tfoot']) {
        editor.model.schema.register(tag, {
          allowIn: 'table',
          isLimit: true,
        });
      }

      editor.model.schema.register('tr', {
        allowIn: ['thead', 'tbody', 'tfoot'],
        isLimit: true,
        allowAttributes: ['scope', 'colspan', 'rowspan', 'headers', 'id', 'class', 'style'],
      });

      for (const cell of ['td', 'th']) {
        editor.model.schema.register(cell, {
          allowIn: 'tr',
          allowContentOf: '$block',
          allowChildren: '$block',
          allowWhere: '$block',
          isLimit: true,
          allowAttributes: ['scope', 'colspan', 'rowspan', 'headers', 'id', 'class', 'style'],
        });
      }

      // Simple view-model converters
      elements.forEach((tag) => {
        editor.conversion.elementToElement({ model: tag, view: tag });
      });

      ['th', 'td', 'tr'].forEach(tag => {
        ['scope', 'colspan', 'rowspan', 'headers', 'id', 'class', 'style'].forEach(attr => {
          editor.conversion.for('upcast').attributeToAttribute({
            view: { name: tag, key: attr },
            model: attr
          });

          editor.conversion.for('downcast').attributeToAttribute({
            model: attr,
            view: attr
          });
        });
      });

      // Deal with faux-button.
      editor.ui.componentFactory.add('cke5_table_fix_dummy', () => null);

    }
  }

  // Expose plugin for Drupal CKEditor 5 loader
  window.CKEditor5 = window.CKEditor5 || {};
  window.CKEditor5.ckeditor5_table_fix = {
    CKEditor5TableFixPlugin: CKEditor5TableFixPlugin
  };
})();
